package gssim.db.mysql;

import gssim.db.dao.data.Experiment;
import gssim.db.find.Finder;
import gssim.db.utils.PreparedStatementAdapter;
import gssim.db.utils.ResultSetAdapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.LogFactory;

public class ExperimentDAO extends AbstractDAO implements
		gssim.db.dao.ExperimentDAO {

	protected ExperimentDAO(Connection conn) throws SQLException {
		super(conn);
		log = LogFactory.getLog(ExperimentDAO.class);
		setRelationName("experiment");
	}

	@Override
	protected PreparedStatementAdapter createDeleteStmt() throws SQLException {
		String sql = "DELETE FROM experiment WHERE PK = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createFindStmt() throws SQLException {
		String sql = "SELECT * FROM experiment WHERE (user_id = ? OR ISNULL(?))"
				+ " AND (commission_date between ? AND ?)"
				+ " AND (name = ? OR ISNULL(?))"
				+ " AND (status = ? OR ISNULL(?))"
				+ " AND (metaexp_id = ? OR ISNULL(?))";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createInsertStmt() throws SQLException {
		String sql = "INSERT INTO experiment(user_id, config_id, commission_date, "
				+ "name, description, status, metaexp_id) VALUES(?, ?, ?, ?, ?, ?, ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql,
						Statement.RETURN_GENERATED_KEYS));

		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createSelectStmt() throws SQLException {
		String sql = "SELECT * FROM experiment WHERE PK = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createUpdateStmt() throws SQLException {
		String sql = "UPDATE experiment SET user_id = ?, config_id = ?, "
				+ "commission_date = ?, name = ?, description = ?,"
				+ "status = ?, metaexp_id = ? WHERE PK = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	public boolean delete(Experiment obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getDeleteStmt();
		stmt.setInt(1, obj.getPK());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("delete");
		} else
			logError("delete", result, obj);

		return (result == 1);
	}

	@Override
	public void fillObjectFromResultSet(Experiment obj, ResultSet resultSet)
			throws SQLException {

		ResultSetAdapter result = (ResultSetAdapter) resultSet;

		Integer pK = result.getIntObj("PK");
		Integer userId = result.getIntObj("user_id");
		Integer configId = result.getIntObj("config_id");
		Timestamp commissionDate = result.getTimestamp("commission_date");
		String name = result.getString("name");
		String description = result.getString("description");
		String status = result.getString("status");
		Integer metaexpId = result.getIntObj("metaexp_id");

		obj.setPK(pK);
		obj.setUserId(userId);
		obj.setConfigId(configId);
		obj.setCommissionDate(commissionDate);
		obj.setName(name);
		obj.setDescription(description);
		obj.setStatus(status);
		obj.setMetaexpId(metaexpId);

	}

	@Override
	public int fillStatementWithObject(PreparedStatement pstmt, Experiment obj,
			int offset) throws SQLException {

		PreparedStatementAdapter stmt = (PreparedStatementAdapter) pstmt;

		stmt.setInt(offset + 1, obj.getUserId());
		stmt.setInt(offset + 2, obj.getConfigId());
		stmt.setTimestamp(offset + 3, obj.getCommissionDate());
		stmt.setString(offset + 4, obj.getName());
		stmt.setString(offset + 5, obj.getDescription());
		stmt.setString(offset + 6, obj.getStatus());
		stmt.setInt(offset + 7, obj.getMetaexpId());
		
		return offset + 7;

	}

	@Override
	public List<Experiment> find(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = getFindStmt();
		List<Experiment> list = new ArrayList<Experiment>();
		
		stmt.setInt(1, finder.getInteger("userId"));
		stmt.setInt(2, finder.getInteger("userId"));
		stmt.setTimestamp(3, finder.getMinTimestamp("commission_date"));
		stmt.setTimestamp(4, finder.getMaxTimestamp("commission_date"));
		stmt.setString(5, finder.getString("name"));
		stmt.setString(6, finder.getString("name"));
		stmt.setString(7, finder.getString("status"));
		stmt.setString(8, finder.getString("status"));
		stmt.setInt(9, finder.getInteger("metaexpId"));
		stmt.setInt(10, finder.getInteger("metaexpId"));
	

		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {
			Experiment obj = new Experiment();
			fillObjectFromResultSet(obj, result);
			list.add(obj);
		}

		return list;

	}

	@Override
	public int insert(Experiment obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		fillStatementWithObject(stmt, obj, 0);

		int result = stmt.executeUpdate();

		if (result == 1) {
			ResultSet gk = stmt.getGeneratedKeys();
			Integer pK = gk.next() ? gk.getInt(1) : -1;
			obj.setPK(pK);

			if (commit)
				commit();

			logSuccess("insert");

		} else
			logError("insert", result, obj);

		return result;
	}

	public int[] insertBatch(List<Experiment> objList, boolean commit)
			throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		for (int i = 0; i < objList.size(); i++) {
			fillStatementWithObject(stmt, objList.get(i), 0);
			stmt.addBatch();
		}

		int[] result = stmt.executeBatch();

		stmt.clearBatch();

		int j = 0;

		while (j < result.length)
			if (result[j] == 1)
				j++;

		if (j == result.length) {

			ResultSet gk = stmt.getGeneratedKeys();
			for (int i = 0; i < objList.size(); i++) {
				Integer PK = gk.next() ? gk.getInt(1) : -1;
				objList.get(i).setPK(PK);
			}

			logSuccess("batch insert", j);

			if (commit)
				commit();

		} else
			logError("batch insert", j);

		return result;
	}

	@Override
	public Experiment select(Experiment obj) throws SQLException {

		PreparedStatementAdapter stmt = getSelectStmt();
		stmt.setInt(1, obj.getPK());
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		result.next();

		boolean first = result.isFirst();
		boolean last = result.isLast();

		if (first && last) {

			fillObjectFromResultSet(obj, result);
			return obj;
		}

		// no user found with specified id
		if (!first && !last) {
			return null;
		}

		// no idea how this may happen (only if id is not primary key)
		throw new SQLException("Select operation in relation "
				+ getRelationName() + " returned more than one record");

	}

	@Override
	public boolean update(Experiment obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getUpdateStmt();

		int offset = fillStatementWithObject(stmt, obj, 0);

		stmt.setInt(offset + 1, obj.getPK());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("update");
		} else
			logError("update", result, obj);

		return (result == 1);
	}

}
